@@ -1,7 +1,6 @@
 """ViSP quality metrics task."""
 from dataclasses import dataclass
 from dataclasses import field
-from typing import List
 
 import numpy as np
 from astropy.time import Time
@@ -18,11 +17,11 @@ from dkist_processing_visp.tasks.visp_base import VispTaskBase
 class _QualityData:
     """Class for storage of Visp quality data."""
 
-    datetimes: List[str] = field(default_factory=list)
-    Q_RMS_noise: List[float] = field(default_factory=list)
-    U_RMS_noise: List[float] = field(default_factory=list)
-    V_RMS_noise: List[float] = field(default_factory=list)
-    intensity_values: List[float] = field(default_factory=list)
+    datetimes: list[str] = field(default_factory=list)
+    I_sensitivity: list[float] = field(default_factory=list)
+    Q_sensitivity: list[float] = field(default_factory=list)
+    U_sensitivity: list[float] = field(default_factory=list)
+    V_sensitivity: list[float] = field(default_factory=list)
 
 
 @dataclass
@@ -30,9 +29,9 @@ class _QualityTaskTypeData:
     """Class for storage of Visp quality task type data."""
 
     quality_task_type: str
-    average_values: List[float] = field(default_factory=list)
-    rms_values_across_frame: List[float] = field(default_factory=list)
-    datetimes: List[str] = field(default_factory=list)
+    average_values: list[float] = field(default_factory=list)
+    rms_values_across_frame: list[float] = field(default_factory=list)
+    datetimes: list[str] = field(default_factory=list)
 
     @property
     def has_values(self) -> bool:
@@ -96,21 +95,25 @@ class VispL1QualityMetrics(VispTaskBase, QualityMixin):
             - Send metrics for storage
 
         """
-        if self.constants.correct_for_polarization:
-            self.compute_polarimetric_metrics()
-
+        self.compute_sensitivity()
         self.compute_noise()
 
-    def compute_polarimetric_metrics(self) -> None:
+    def compute_sensitivity(self) -> None:
         """Compute RMS noise and sensitivity estimate for L1 Visp frames."""
         with self.apm_processing_step("Calculating polarization metrics"):
             all_datetimes = []
-            all_Q_RMS_noise = []
-            all_U_RMS_noise = []
-            all_V_RMS_noise = []
-            all_pol_sens_values = []
+            all_I_sensitivity = []
+            all_Q_sensitivity = []
+            all_U_sensitivity = []
+            all_V_sensitivity = []
             for map_scan in range(1, self.constants.num_map_scans + 1):
                 polarization_data = _QualityData()
+                poldata_noise_list_list = [
+                    polarization_data.I_sensitivity,
+                    polarization_data.Q_sensitivity,
+                    polarization_data.U_sensitivity,
+                    polarization_data.V_sensitivity,
+                ]
                 for step in range(0, self.constants.num_raster_steps):
 
                     # grab stokes I data
@@ -126,52 +129,53 @@ class VispL1QualityMetrics(VispTaskBase, QualityMixin):
                             cls=L1QualityFitsAccess,
                         )
                     )
-                    stokesI_data = stokesI_frame.data
+                    stokesI_med = np.nanmedian(stokesI_frame.data)
                     polarization_data.datetimes.append(Time(stokesI_frame.time_obs).mjd)
-                    polarization_data.intensity_values.append(np.max(stokesI_data))
 
                     # grab other stokes data and find and store RMS noise
                     for stokes_param, data_list in zip(
-                        ("Q", "U", "V"),
-                        (
-                            polarization_data.Q_RMS_noise,
-                            polarization_data.U_RMS_noise,
-                            polarization_data.V_RMS_noise,
-                        ),
+                        ("I", "Q", "U", "V"), poldata_noise_list_list
                     ):
-                        stokes_frame = next(
-                            self.fits_data_read_fits_access(
-                                tags=[
-                                    VispTag.output(),
-                                    VispTag.frame(),
-                                    VispTag.raster_step(step),
-                                    VispTag.map_scan(map_scan),
-                                    VispTag.stokes(stokes_param),
-                                ],
-                                cls=L1QualityFitsAccess,
+                        try:
+                            stokes_frame = next(
+                                self.fits_data_read_fits_access(
+                                    tags=[
+                                        VispTag.output(),
+                                        VispTag.frame(),
+                                        VispTag.raster_step(step),
+                                        VispTag.map_scan(map_scan),
+                                        VispTag.stokes(stokes_param),
+                                    ],
+                                    cls=L1QualityFitsAccess,
+                                )
                             )
-                        )
-                        # find Stokes RMS noise
-                        data_list.append(np.std(stokes_frame.data / stokesI_data))
+                        except StopIteration:
+                            # This stokes parameter doesn't exist. No big deal.
+                            continue
+
+                        # compute sensitivity for this Stokes parameter
+                        data_list.append(np.std(stokes_frame.data) / stokesI_med)
 
                 all_datetimes.append(Time(np.mean(polarization_data.datetimes), format="mjd").isot)
-                all_Q_RMS_noise.append(np.average(polarization_data.Q_RMS_noise))
-                all_U_RMS_noise.append(np.average(polarization_data.U_RMS_noise))
-                all_V_RMS_noise.append(np.average(polarization_data.V_RMS_noise))
-                # find the polarimetric sensitivity of this map_scan (smallest intensity signal measured)
-                polarimetric_sensitivity = 1 / np.max(polarization_data.intensity_values)
-                all_pol_sens_values.append(polarimetric_sensitivity)
+                for target, source in zip(
+                    [all_I_sensitivity, all_Q_sensitivity, all_U_sensitivity, all_V_sensitivity],
+                    poldata_noise_list_list,
+                ):
+                    if not source:
+                        # Empty list means there are no data for this Stokes parameter
+                        continue
+                    target.append(np.mean(source))
 
         with self.apm_step("Sending lists for storage"):
             for stokes_index, stokes_noise in zip(
-                ("Q", "U", "V"), (all_Q_RMS_noise, all_U_RMS_noise, all_V_RMS_noise)
+                ("I", "Q", "U", "V"),
+                (all_I_sensitivity, all_Q_sensitivity, all_U_sensitivity, all_V_sensitivity),
             ):
-                self.quality_store_polarimetric_noise(
+                if not stokes_noise:
+                    continue
+                self.quality_store_sensitivity(
                     stokes=stokes_index, datetimes=all_datetimes, values=stokes_noise
                 )
-            self.quality_store_polarimetric_sensitivity(
-                datetimes=all_datetimes, values=all_pol_sens_values
-            )
 
     def compute_noise(self):
         """Compute noise in data."""
