@@ -1,7 +1,5 @@
 """ViSP solar calibration task."""
 import logging
-from typing import List
-from typing import Tuple
 
 import numpy as np
 import peakutils
@@ -50,7 +48,7 @@ class SolarCalibration(
         For each beam.
 
             For each modstate:
-                - Do dark, lamp, and geometric corrections
+                - Do dark, background, lamp, and geometric corrections
                 - Compute the characteristic spectra
                 - Re-apply the spectral curvature to the characteristic spectra
                 - Re-apply angle and state offset distortions to the characteristic spectra
@@ -71,27 +69,64 @@ class SolarCalibration(
 
                 with self.apm_processing_step(f"Computing characteristic spectra for {apm_str}"):
                     char_spec = self.compute_characteristic_spectra(beam=beam, modstate=modstate)
+                    self.intermediate_frame_helpers_write_arrays(
+                        arrays=char_spec, beam=beam, modstate=modstate, task="SC_DEBUG_CHAR_SPEC"
+                    )
 
-                with self.apm_processing_step(f"Reshifting characteristic spectra for {apm_str}"):
-                    reshifted_char_spec = self.reshift_characteristic_spectra(
+                with self.apm_processing_step(
+                    f"Refining characteristic spectral shifts for {apm_str}"
+                ):
+                    reshifted_char_spec = self.refine_gain_shifts(
                         char_spec=char_spec, beam=beam, modstate=modstate
                     )
+                    self.intermediate_frame_helpers_write_arrays(
+                        arrays=reshifted_char_spec,
+                        beam=beam,
+                        modstate=modstate,
+                        task="SC_DEBUG_CHAR_SPEC_SHIFT",
+                    )
+
+                with self.apm_processing_step(f"Removing solar signal from {apm_str}"):
+                    gain = self.remove_solar_signal(
+                        char_solar_spectra=reshifted_char_spec, beam=beam, modstate=modstate
+                    )
+                    self.intermediate_frame_helpers_write_arrays(
+                        arrays=gain,
+                        beam=beam,
+                        modstate=modstate,
+                        task="SC_DEBUG_PERFECT_GAIN",
+                    )
+
+                with self.apm_processing_step(f"Reshifting gain spectra for {apm_str}"):
+                    spec_shift = self.intermediate_frame_helpers_load_spec_shift(beam=beam)
+                    reshifted_gain = next(
+                        self.corrections_remove_spec_geometry(
+                            arrays=gain, spec_shift=-1 * spec_shift
+                        )
+                    )
+                    self.intermediate_frame_helpers_write_arrays(
+                        arrays=reshifted_gain,
+                        beam=beam,
+                        modstate=modstate,
+                        task="SC_DEBUG_NEW_GAIN_SPEC_SHIFT",
+                    )
 
                 with self.apm_processing_step(
                     f"Re-distorting characteristic spectra for {apm_str}"
                 ):
-                    distorted_char_spec = self.distort_characteristic_spectra(
-                        char_spec=reshifted_char_spec, beam=beam, modstate=modstate
+                    distorted_gain = self.distort_characteristic_spectra(
+                        char_spec=reshifted_gain, beam=beam, modstate=modstate
                     )
-
-                with self.apm_processing_step(f"Removing solar signal from {apm_str}"):
-                    final_gain = self.remove_solar_signal(
-                        char_solar_spectra=distorted_char_spec, beam=beam, modstate=modstate
+                    self.intermediate_frame_helpers_write_arrays(
+                        arrays=distorted_gain,
+                        beam=beam,
+                        modstate=modstate,
+                        task="SC_DEBUG_NEW_GAIN_SHIFT_DISTORT",
                     )
 
                 with self.apm_writing_step(f"Writing solar gain for {apm_str}"):
                     self.write_solar_gain_calibration(
-                        gain_array=final_gain, beam=beam, modstate=modstate
+                        gain_array=distorted_gain, beam=beam, modstate=modstate
                     )
 
         with self.apm_processing_step("Computing and logging quality metrics"):
@@ -107,30 +142,6 @@ class SolarCalibration(
                 task_type="SOLAR_GAIN", total_frames=no_of_raw_solar_frames
             )
 
-    def dark_only_modstate_data(self, beam: int, modstate: int) -> np.ndarray:
-        """
-        Array for a single beam/modstate that has ONLY had the dark signal removed.
-
-        Parameters
-        ----------
-        beam : int
-            The beam number for this array
-
-        modstate : int
-            The modulator state for this array
-
-
-        Returns
-        -------
-        np.ndarray
-            Array with dark signal removed
-
-        """
-        array_generator = self.intermediate_frame_helpers_load_intermediate_arrays(
-            beam=beam, task="SC_DARK_ONLY", modstate=modstate
-        )
-        return next(array_generator)
-
     def unshifted_geo_corrected_modstate_data(self, beam: int, modstate: int) -> np.ndarray:
         """
         Array for a single beam/modstate that has dark, lamp, angle, and state offset corrections.
@@ -184,9 +195,7 @@ class SolarCalibration(
 
         At two intermediate points the current arrays are saved because they'll be needed by various helpers:
 
-        SC_DARK_ONLY - The solar gain arrays with only a dark correction. These arrays will become the true Solar Gain
-                       images after the solar spectral signature is removed. We do this so the Solar Gain images also
-                       contain the lamp gain signal.
+        SC_BG_ONLY - The solar gain arrays with only a dark and background correction.
 
         SC_GEO_NOSHIFT - The solar gain arrays after dark, lamp, angle, and state offset correction. In other words,
                          they do not have spectral curvature removed. These are used to reshift the characteristic
@@ -213,7 +222,18 @@ class SolarCalibration(
             except StopIteration:
                 raise ValueError(f"No matching dark found for {exp_time = } s")
 
-            logging.info(f"Doing dark, lamp, and geo corrections for {beam=} and {modstate=}")
+            try:
+                background_array = self.intermediate_frame_helpers_load_background_array(beam=beam)
+            except StopIteration:
+                if self.constants.correct_for_polarization:
+                    raise ValueError(f"No matching background light found for {beam = }")
+
+                logging.info("Skipping background light correction")
+                background_array = np.zeros(dark_array.shape)
+
+            logging.info(
+                f"Doing dark, background, lamp, and geo corrections for {beam=} and {modstate=}"
+            )
             ## Load frames
             input_solar_arrays = self.input_frame_loaders_solar_gain_array_generator(
                 beam=beam, modstate=modstate, exposure_time=exp_time
@@ -226,18 +246,27 @@ class SolarCalibration(
             avg_solar_array = average_numpy_arrays(input_solar_arrays)
 
             ## Dark correction
-            dark_corrected_solar_array = next(
-                subtract_array_from_arrays(arrays=avg_solar_array, array_to_subtract=dark_array)
+            dark_corrected_solar_array = subtract_array_from_arrays(
+                arrays=avg_solar_array, array_to_subtract=dark_array
             )
+
+            ## Residual background correction
+            background_corrected_solar_array = next(
+                subtract_array_from_arrays(dark_corrected_solar_array, background_array)
+            )
+
             # Save the only-dark-corr because this will be used to make the final Solar Gain object
             self.intermediate_frame_helpers_write_arrays(
-                arrays=dark_corrected_solar_array, beam=beam, modstate=modstate, task="SC_DARK_ONLY"
+                arrays=background_corrected_solar_array,
+                beam=beam,
+                modstate=modstate,
+                task="SC_BG_ONLY",
             )
 
             ## Lamp correction
             lamp_corrected_solar_array = next(
                 divide_arrays_by_array(
-                    arrays=dark_corrected_solar_array, array_to_divide_by=lamp_array
+                    arrays=background_corrected_solar_array, array_to_divide_by=lamp_array
                 )
             )
 
@@ -255,6 +284,9 @@ class SolarCalibration(
             self.intermediate_frame_helpers_write_arrays(
                 arrays=geo_corrected_array, beam=beam, modstate=modstate, task="SC_GEO_NOSHIFT"
             )
+            # re_distorted_array = self.distort_characteristic_spectra(geo_corrected_array, beam=beam, modstate=modstate)
+            # self.intermediate_frame_helpers_write_arrays(arrays=re_distorted_array, beam=beam, modstate=modstate,
+            #                                              task="SC_DEBUG_REDISTORTED_OBS")
 
             # Now finish the spectral shift correction
             spectral_corrected_array = next(
@@ -266,7 +298,9 @@ class SolarCalibration(
 
     def compute_characteristic_spectra(self, beam: int, modstate: int) -> np.ndarray:
         """
-        Compute the characteristic spectra via a moving average along the slit, identify and ignore hairlines.
+        Compute the characteristic spectrum by taking the median of all spatial positions.
+
+        First identify and ignore hairlines, which might not be necessary.
 
         Parameters
         ----------
@@ -283,10 +317,10 @@ class SolarCalibration(
             Characteristic spectra array
         """
         spectral_avg_window = self.parameters.solar_spectral_avg_window
-        hairline_fraction = self.parameters.solar_hairline_fraction
+        hairline_fraction = self.parameters.gain_hairline_fraction
 
         logging.info(
-            f"Computing characteristic spectra for {beam=} and {modstate=} using {spectral_avg_window=} and {hairline_fraction=}"
+            f"Computing characteristic spectra for {beam = } and {modstate = } with {spectral_avg_window = } and {hairline_fraction = }"
         )
         full_spectra = self.geo_corrected_modstate_data(beam=beam, modstate=modstate)
 
@@ -296,22 +330,20 @@ class SolarCalibration(
         lidx = np.where(ratio < hairline_fraction)
 
         # Replace identified hairlines with data from the median spectrum
+        # This might all be unnecessary
         full_spectra[lidx] = med_spec[lidx[0]]
 
-        # Compute characteristic spectra with a moving Gaussian average
+        # Actually compute the median spectrum and explode it across the full shape
         char_spec = spnd.gaussian_filter1d(full_spectra, spectral_avg_window, axis=1)
 
         return char_spec
 
-    def reshift_characteristic_spectra(
-        self, char_spec: np.ndarray, beam: int, modstate: int
-    ) -> np.ndarray:
+    def refine_gain_shifts(self, char_spec: np.ndarray, beam: int, modstate: int) -> np.ndarray:
         """
-        Re-apply the spectral curvature to the characteristic spectra.
+        Refine the spectral shifts when matching characteristic spectra to the rectified input spectra.
 
-        Note: This function re-fits the shifts instead of using the pre-computed shifts, which seems a little weird, but is
-        empirically required. Importantly, the fit metric for this re-shift is different than that used to compute the
-        initial shift. Here we actually minimize residuals in the final gain image.
+        An important detail of this functino is that the goodness of fit metric is the final gain image (i.e., raw
+        input with solar spectrum removed). We minimize the residuals in the gain image.
 
         Parameters
         ----------
@@ -328,16 +360,11 @@ class SolarCalibration(
         Returns
         -------
         np.ndarray
-            Re-shifted characteristic spectra array
+            Characteristic spectra array with refined spectral shifts
         """
-        # Grab initial guesses
-        spec_shifts = self.intermediate_frame_helpers_load_spec_shift(beam=beam)
-        num_spec = spec_shifts.size
-
-        # Grab unshifted spectra that will be the shift target
-        unshifted_raw_spec = self.unshifted_geo_corrected_modstate_data(
-            beam=beam, modstate=modstate
-        )
+        # Grab rectified input spectra that will be the shift target
+        target_spectra = self.geo_corrected_modstate_data(beam=beam, modstate=modstate)
+        num_spec = target_spectra.shape[1]
 
         logging.info(f"Computing line zones for {beam=} and {modstate=}")
         zone_kwargs = {
@@ -353,18 +380,13 @@ class SolarCalibration(
             raise ValueError(f"No zones found for {beam=} and {modstate=}")
 
         reshift_char_spec = np.zeros(char_spec.shape)
-        logging.info(f"Fitting reshifts for {beam=} and {modstate=}")
+        logging.info(f"Refining shifts for {beam=} and {modstate=}")
         for i in range(num_spec):
-            ref_spec = unshifted_raw_spec[:, i]
+            ref_spec = target_spectra[:, i]
             spec = char_spec[:, i]
-            shift = SolarCalibration.reshift(spec, ref_spec, zones=zones, x_init=spec_shifts[i])
+            shift = SolarCalibration.refine_shift(spec, ref_spec, zones=zones, x_init=0.0)
             reshift_char_spec[:, i] = spnd.shift(char_spec[:, i], shift, mode="reflect")
 
-        logging.info(f"Normalizing spectra for {beam=} and {modstate=}")
-        raw_meds = np.nanmedian(unshifted_raw_spec, axis=0)
-        char_meds = np.nanmedian(reshift_char_spec, axis=0)
-        reshift_char_spec *= (raw_meds / char_meds)[None, :]
-
         return reshift_char_spec
 
     def distort_characteristic_spectra(
@@ -408,6 +430,8 @@ class SolarCalibration(
         """
         Remove the distorted characteristic solar spectra from the original spectra.
 
+        This also has the effect of normalizing the gain image.
+
         Parameters
         ----------
         char_solar_spectra : np.ndarray
@@ -427,9 +451,9 @@ class SolarCalibration(
 
         """
         logging.info(f"Removing characteristic solar spectra from {beam=} and {modstate=}")
-        raw_gain = self.dark_only_modstate_data(beam=beam, modstate=modstate)
+        input_gain = self.geo_corrected_modstate_data(beam=beam, modstate=modstate)
 
-        final_gain = raw_gain / char_solar_spectra
+        final_gain = input_gain / char_solar_spectra
 
         return final_gain
 
@@ -481,7 +505,7 @@ class SolarCalibration(
         bg_order: int = 22,
         normalization_percentile: int = 99,
         rel_height=0.97,
-    ) -> List[Tuple[int, int]]:
+    ) -> list[tuple[int, int]]:
         """
         Identify spectral regions around strong spectra features.
 
@@ -541,7 +565,7 @@ class SolarCalibration(
         return list(zip(rips, lips))
 
     @staticmethod
-    def identify_overlapping_zones(rips: np.ndarray, lips: np.ndarray) -> List[int]:
+    def identify_overlapping_zones(rips: np.ndarray, lips: np.ndarray) -> list[int]:
         """
         Identify line zones that overlap with other zones. Any overlap greater than 1 pixel is flagged.
 
@@ -580,11 +604,11 @@ class SolarCalibration(
         return ranges_to_remove
 
     @staticmethod
-    def reshift(
-        spec: np.ndarray, target_spec: np.ndarray, zones: List[Tuple[int, int]], x_init: float
+    def refine_shift(
+        spec: np.ndarray, target_spec: np.ndarray, zones: list[tuple[int, int]], x_init: float
     ) -> float:
         """
-        Compute the shift for a single spatial position back from rectified spectra to the original (curved) position.
+        Refine the shift for a single spatial position back to the rectified input spectra.
 
         Line zones are used to increase the SNR of the chisq and the final shift is the mean of the shifts computed
         for each zone.
@@ -626,7 +650,7 @@ class SolarCalibration(
 
     @staticmethod
     def shift_func(
-        par: List[float], ref_spec: np.ndarray, spec: np.ndarray, idx: np.ndarray
+        par: list[float], ref_spec: np.ndarray, spec: np.ndarray, idx: np.ndarray
     ) -> float:
         """
         Non-chisq based goodness of fit calculator for computing spectral shifts.
