@@ -4,6 +4,7 @@ import logging
 import os
 import sys
 from dataclasses import asdict
+from datetime import datetime
 from pathlib import Path
 from random import randint
 
@@ -19,6 +20,7 @@ from dkist_processing_common.tasks.mixin.quality import QualityMixin
 from dkist_processing_visp.models.tags import VispTag
 from dkist_processing_visp.parsers.visp_l0_fits_access import VispL0FitsAccess
 from dkist_processing_visp.tasks.assemble_movie import AssembleVispMovie
+from dkist_processing_visp.tasks.background_light import BackgroundLightCalibration
 from dkist_processing_visp.tasks.dark import DarkCalibration
 from dkist_processing_visp.tasks.geometric import GeometricCalibration
 from dkist_processing_visp.tasks.instrument_polarization import InstrumentPolarizationCalibration
@@ -33,11 +35,13 @@ from dkist_processing_visp.tasks.solar import SolarCalibration
 from dkist_processing_visp.tasks.visp_base import VispTaskBase
 from dkist_processing_visp.tasks.write_l1 import VispWriteL1Frame
 from dkist_processing_visp.tests.conftest import VispTestingParameters
+from dkist_processing_visp.tests.e2e_helpers import LoadBackgroundCal
 from dkist_processing_visp.tests.e2e_helpers import LoadDarkCal
 from dkist_processing_visp.tests.e2e_helpers import LoadGeometricCal
 from dkist_processing_visp.tests.e2e_helpers import LoadInstPolCal
 from dkist_processing_visp.tests.e2e_helpers import LoadLampCal
 from dkist_processing_visp.tests.e2e_helpers import LoadSolarCal
+from dkist_processing_visp.tests.e2e_helpers import SaveBackgroundCal
 from dkist_processing_visp.tests.e2e_helpers import SaveDarkCal
 from dkist_processing_visp.tests.e2e_helpers import SaveGeometricCal
 from dkist_processing_visp.tests.e2e_helpers import SaveInstPolCal
@@ -57,7 +61,6 @@ QRM = False
 try:
     from quality_report_maker.libraries import report
     from quality_report_maker.libraries.json_encoder import datetime_json_object_hook
-    from quality_report_maker.libraries.json_encoder import DatetimeEncoder
 
     QRM = True
 except ModuleNotFoundError:
@@ -68,6 +71,18 @@ if QRM:
     plt.ioff()
 
 
+class DatetimeEncoder(json.JSONEncoder):
+    # Copied from quality_report_maker
+    """
+    A JSON encoder which encodes datetime(s) as iso formatted strings.
+    """
+
+    def default(self, obj):
+        if isinstance(obj, datetime):
+            return {"iso_date": obj.isoformat("T")}
+        return super().default(obj)
+
+
 class Translate122To214L0(WorkflowTaskBase):
     def run(self) -> None:
         raw_dir = Path(self.scratch.scratch_base_path) / f"VISP{self.recipe_run_id:03n}"
@@ -100,15 +115,15 @@ class Translate122To214L0(WorkflowTaskBase):
 
 class CreateInputDatasetParameterDocument(WorkflowTaskBase):
     def run(self) -> None:
-        doc_path = self.scratch.workflow_base_path / "dataset_doc.json"
+        doc_path = self.scratch.workflow_base_path / "input_dataset_parameters.json"
         with open(doc_path, "w") as f:
-            f.write(json.dumps(self.input_dataset_document_with_simple_parameters))
-        self.tag(doc_path, VispTag.input_dataset())
+            f.write(json.dumps(self.input_dataset_document_simple_parameters_part))
+        self.tag(doc_path, VispTag.input_dataset_parameters())
         logging.info(f"Wrote input dataset doc to {doc_path}")
 
     @property
-    def input_dataset_document_with_simple_parameters(self):
-        dataset_doc_dict = dict(parameters=[])
+    def input_dataset_document_simple_parameters_part(self):
+        parameters_list = []
         value_id = randint(1000, 2000)
         for pn, pv in asdict(VispTestingParameters()).items():
             values = [
@@ -119,9 +134,9 @@ class CreateInputDatasetParameterDocument(WorkflowTaskBase):
                 }
             ]
             parameter = {"parameterName": pn, "parameterValues": values}
-            dataset_doc_dict["parameters"] += [parameter]
+            parameters_list.append(parameter)
 
-        return dataset_doc_dict
+        return parameters_list
 
 
 def tag_inputs_task(suffix: str):
@@ -189,19 +204,18 @@ class SubmitAndExposeQuality(VispSubmitQuality):
         super().run()
 
         # Now regenerate the report str and save it
-        if QRM:
-            logging.info("Building quality report for save to disk")
-            report_str = self.quality_build_report(polcal_label_list=self.polcal_label_list)
+        logging.info("Building quality report for save to disk")
+        report_str = self.quality_build_report(polcal_label_list=self.polcal_label_list)
 
-            doc_path = self.scratch.workflow_base_path / "quality_report.json"
-            report_container = {
-                "datasetId": self.constants.dataset_id,
-                "qualityReport": json.dumps(report_str, cls=DatetimeEncoder),
-            }
-            json_str = json.dumps(report_container)
-            with open(doc_path, "w") as f:
-                f.write(json_str)
-            logging.info(f"Wrote report to {doc_path}")
+        doc_path = self.scratch.workflow_base_path / "quality_report.json"
+        report_container = {
+            "datasetId": self.constants.dataset_id,
+            "qualityReport": json.dumps(report_str, cls=DatetimeEncoder),
+        }
+        json_str = json.dumps(report_container)
+        with open(doc_path, "w") as f:
+            f.write(json_str)
+        logging.info(f"Wrote report to {doc_path}")
 
 
 class ValidateL1Output(VispTaskBase):
@@ -268,6 +282,7 @@ def main(
     skip_translation: bool = False,
     only_translate: bool = False,
     load_dark: bool = False,
+    load_background: bool = False,
     load_lamp: bool = False,
     load_geometric: bool = False,
     load_solar: bool = False,
@@ -300,6 +315,12 @@ def main(
             manual_processing_run.run_task(task=DarkCalibration)
             manual_processing_run.run_task(task=SaveDarkCal)
 
+        if load_background:
+            manual_processing_run.run_task(task=LoadBackgroundCal)
+        else:
+            manual_processing_run.run_task(task=BackgroundLightCalibration)
+            manual_processing_run.run_task(task=SaveBackgroundCal)
+
         if load_lamp:
             manual_processing_run.run_task(task=LoadLampCal)
         else:
@@ -371,6 +392,12 @@ if __name__ == "__main__":
         help="Load dark calibration from previously saved run",
         action="store_true",
     )
+    parser.add_argument(
+        "-B",
+        "--load-background",
+        help="Load background light calibration from previously saved run",
+        action="store_true",
+    )
     parser.add_argument(
         "-L",
         "--load-lamp",
@@ -405,6 +432,7 @@ if __name__ == "__main__":
             skip_translation=args.skip_translation,
             only_translate=args.only_translate,
             load_dark=args.load_dark,
+            load_background=args.load_background,
             load_lamp=args.load_lamp,
             load_geometric=args.load_geometric,
             load_solar=args.load_solar,
